"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createLongpollClient = createLongpollClient;
exports.SERVER = void 0;

var _debug = _interopRequireDefault(require("debug"));

var _nodeFetch = _interopRequireDefault(require("node-fetch"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const debug = (0, _debug.default)('superbot:network:longpoll');

function loop(request, handler, errorHandler) {
  debug('create loop');
  let work = true;

  async function totalRun() {
    let it = 0;

    while (work) {
      it++;
      debug('iteration', it);

      try {
        const result = await request();

        if (work) {
          handler(result);
        }
      } catch (error) {
        debug('iteration error', it, error);

        if (work) {
          errorHandler(error);
        }
      }
    }
  }

  return function start() {
    debug('start loop');
    setTimeout(totalRun, 1);
    return function stop() {
      debug('stop loop');
      work = false;
    };
  };
}

function createLongpollClient(server, timeout = 10000) {
  const errorHandler = error => debug('longpoll error', error);

  return function startClient(botToken, handler) {
    // get updates
    const request = () => (0, _nodeFetch.default)(`${server}/bot${botToken}/getUpdates?timeout=${timeout}`, {
      timeout
    }).then(response => response.json()).then(answer => answer.ok ? answer.result : []);

    const updateHandler = updates => {
      updates.forEach(handler);
    };

    const start = loop(request, updateHandler, errorHandler);
    return start();
  };
}

const SERVER = {
  DEFAULT: 'https://api.telegram.org'
};
exports.SERVER = SERVER;